from typing import Optional, List
from fastapi import APIRouter, Depends, HTTPException, Query
from sqlalchemy.orm import Session
import math

from app.crud import todo as todo_crud
from app.db.session import get_db
from app.models.todo import Priority
from app.schemas.todo import (
    Todo,
    TodoCreate,
    TodoUpdate,
    TodoListResponse,
    SubtaskCreate,
)

router = APIRouter(prefix="/todos", tags=["todos"])


@router.get("/", response_model=TodoListResponse)
def read_todos(
    page: int = Query(1, ge=1, description="Page number"),
    per_page: int = Query(10, ge=1, le=100, description="Items per page"),
    completed: Optional[bool] = Query(None, description="Filter by completion status"),
    priority: Optional[Priority] = Query(None, description="Filter by priority"),
    search: Optional[str] = Query(None, description="Search in title and description"),
    category_id: Optional[int] = Query(None, description="Filter by category ID"),
    db: Session = Depends(get_db),
):
    skip = (page - 1) * per_page
    todos, total = todo_crud.get_todos(
        db,
        skip=skip,
        limit=per_page,
        completed=completed,
        priority=priority,
        search=search,
        category_id=category_id,
    )

    total_pages = math.ceil(total / per_page)

    return TodoListResponse(
        items=todos,
        total=total,
        page=page,
        per_page=per_page,
        has_next=page < total_pages,
        has_prev=page > 1,
    )


@router.post("/", response_model=Todo)
def create_todo(todo: TodoCreate, db: Session = Depends(get_db)):
    return todo_crud.create_todo(db=db, todo=todo)


@router.get("/{todo_id}", response_model=Todo)
def read_todo(todo_id: int, db: Session = Depends(get_db)):
    db_todo = todo_crud.get_todo(db, todo_id=todo_id)
    if db_todo is None:
        raise HTTPException(status_code=404, detail="Todo not found")
    return db_todo


@router.put("/{todo_id}", response_model=Todo)
def update_todo(todo_id: int, todo_update: TodoUpdate, db: Session = Depends(get_db)):
    db_todo = todo_crud.update_todo(db, todo_id=todo_id, todo_update=todo_update)
    if db_todo is None:
        raise HTTPException(status_code=404, detail="Todo not found")
    return db_todo


@router.delete("/{todo_id}")
def delete_todo(todo_id: int, db: Session = Depends(get_db)):
    success = todo_crud.delete_todo(db, todo_id=todo_id)
    if not success:
        raise HTTPException(status_code=404, detail="Todo not found")
    return {"message": "Todo deleted successfully"}


@router.post("/{todo_id}/subtasks", response_model=Todo)
def create_subtask(todo_id: int, subtask: SubtaskCreate, db: Session = Depends(get_db)):
    """Create a subtask for a specific todo."""
    db_subtask = todo_crud.create_subtask(db, parent_id=todo_id, subtask=subtask)
    if db_subtask is None:
        raise HTTPException(status_code=404, detail="Parent todo not found")
    return db_subtask


@router.get("/{todo_id}/subtasks", response_model=List[Todo])
def get_subtasks(todo_id: int, db: Session = Depends(get_db)):
    """Get all subtasks for a specific todo."""
    # First check if parent todo exists
    parent_todo = todo_crud.get_todo(db, todo_id=todo_id)
    if parent_todo is None:
        raise HTTPException(status_code=404, detail="Todo not found")

    subtasks = todo_crud.get_subtasks(db, parent_id=todo_id)
    return subtasks


@router.put("/subtasks/{subtask_id}/move", response_model=Todo)
def move_subtask(
    subtask_id: int, new_parent_id: Optional[int] = None, db: Session = Depends(get_db)
):
    """Move a subtask to a different parent or make it a main todo."""
    moved_subtask = todo_crud.move_subtask(
        db, subtask_id=subtask_id, new_parent_id=new_parent_id
    )
    if moved_subtask is None:
        raise HTTPException(
            status_code=400,
            detail="Cannot move subtask: subtask not found, invalid parent, or would create a cycle",
        )
    return moved_subtask
